import os
from enum import Enum, unique
from pathlib import Path
from typing import Annotated

import typer
from rich.console import Console
from rich.progress import track

from fastapi_translations import printer
from fastapi_translations.git import FastAPIGitDocs
from fastapi_translations.models import DocFile, Summary

console = Console()

app = typer.Typer(rich_markup_mode="rich")


@unique
class Languages(Enum):
    az = "az"
    bn = "bn"
    de = "de"
    em = "em"
    es = "es"
    fa = "fa"
    fr = "fr"
    he = "he"
    hu = "hu"
    id = "id"
    it = "it"
    ja = "ja"
    ko = "ko"
    pl = "pl"
    pt = "pt"
    ru = "ru"
    tr = "tr"
    uk = "uk"
    ur = "ur"
    vi = "vi"
    yo = "yo"
    zh = "zh"
    zh_hant = "zh-hant"


@app.command("report")
def report(
    lang: Annotated[
        Languages,
        typer.Option(
            ...,
            "--language", "-l",
            help="The language to check for translations report"
        ),
    ],
    save_csv: Annotated[
        bool,
        typer.Option(
            "--csv", "-c",
            help="Save all missing and outdated translations to a csv file"
        )
    ] = False
) -> None:
    """Generate a report for the translated docs"""
    base_docs_path = Path("docs")
    en_docs_path = Path("docs/en")

    summary = Summary(lang=lang.value)
    console.clear()
    git = FastAPIGitDocs()

    for root, _, files in track(
        os.walk(en_docs_path),
        "🚶 Walking through 📂 docs looking for 🔠 translations",
    ):
        for file in files:
            if file.endswith(".md"):
                file_relative_path = os.path.relpath(
                    os.path.join(root, file), en_docs_path
                )
                translated_path = os.path.join(
                    base_docs_path, lang.value, file_relative_path
                )
                translation_exists = os.path.exists(translated_path)

                original_doc_date = git.get_commit_date_for(
                    os.path.join(root, file)
                )
                translation_is_outdated = False
                translated_date = git.get_commit_date_for(translated_path)

                if translated_date and original_doc_date:
                    translation_is_outdated = translated_date > original_doc_date

                doc = DocFile(
                    translation_lang=lang.value,
                    original_file=os.path.join(root, file),
                    original_commit=original_doc_date,
                    translation_file=translated_path,
                    translation_exists=translation_exists,
                    translation_commit=translated_date,
                    translation_is_outdated=translation_is_outdated
                )
                summary.append_file(doc)
    printer.print_table(summary, console, 10)

    if save_csv:
        printer.print_to_csv(summary)


def main() -> None:
    app()



import csv

from rich.console import Console
from rich.table import Table

from fastapi_translations.models import DocFile, Summary


def print_table(summary: Summary, console: Console, table_size: int = 10) -> None:
    console.clear()

    table_stats = Table(title="Stats")
    table_stats.add_column("📂 FastAPI docs")
    table_stats.add_column("😢 Missing translations")
    table_stats.add_column("📅 Oudated translations")

    table_docs = Table()
    table_docs.add_column("Count")
    table_docs.add_row(f"{summary.files_analyzed}")

    table_missing = Table()
    table_missing.add_column("Count", justify="center", style="bold cyan")
    table_missing.add_column("Percentage", justify="center", style="bold cyan")
    table_missing.add_row(
        f"{summary.files_missing_translation}",
        f"% {summary.percentage_missing_translation:.2f}"
    )

    table_outdated = Table()
    table_outdated.add_column("Count", justify="center", style="bold cyan")
    table_outdated.add_column("Percentage", justify="center", style="bold cyan")

    table_outdated.add_row(
        f"{summary.files_outdated}",
        f"% {summary.percentage_outdated_translation:.2f}"
    )

    table_stats.add_row(table_docs, table_missing, table_outdated)
    console.print(table_stats)
    console.line()

    table_files = Table(title="🆘 Need help on")
    table_files.add_column(f"First {table_size} missing")
    table_files.add_column(f"First {table_size} outdated")

    table_first_outdated = Table()
    table_first_outdated.add_column("📂 File", justify="left", style="bold cyan")

    for file in summary.first_outdated_files(table_size):
        table_first_outdated.add_row(f"📂 {file.original_file}")

    table_first_missing = Table()
    table_first_missing.add_column("📂 File", justify="left", style="bold cyan")

    for file in summary.first_missing_translation_files(table_size):
        table_first_missing.add_row(f"📂 {file.original_file}")

    table_files.add_row(table_first_missing, table_first_outdated)
    console.print(table_files)


def print_to_csv(summary: Summary) -> None:
    header = DocFile.model_fields.keys()
    with open(f"fastapi-translations-lang-{summary.lang}.csv", "w", newline="") as f:
        writer = csv.DictWriter(f, header)
        writer.writeheader()
        writer.writerows([f.model_dump() for f in summary.files])



from datetime import datetime

from pydantic import BaseModel, computed_field


class DocFile(BaseModel):
    official_lang: str = "en"
    translation_lang: str
    original_file: str
    original_commit: datetime | None
    translation_file: str | None = None
    translation_exists: bool
    translation_commit: datetime | None
    translation_is_outdated: bool


class Summary(BaseModel):
    lang: str
    files_analyzed: int = 0
    files_translated: int = 0
    files_outdated: int = 0
    files_missing_translation: int = 0
    files: list[DocFile] = []

    @computed_field  # type: ignore
    @property
    def percentage_translated(self) -> float:
        try:
            return (
                100 * float(self.files_translated) / float(self.files_analyzed)
            )
        except Exception:
            return 0.0

    @computed_field  # type: ignore
    @property
    def percentage_missing_translation(self) -> float:
        try:
            return (
                100
                * float(self.files_missing_translation)
                / float(self.files_analyzed)
            )
        except Exception:
            return 0.0

    @computed_field  # type: ignore
    @property
    def percentage_outdated_translation(self) -> float:
        try:
            return 100 * float(self.files_outdated) / float(self.files_analyzed)
        except Exception:
            return 0.0

    def append_file(self, doc: DocFile) -> None:
        self.files.append(doc)
        self.files_analyzed += 1

        if doc.translation_exists:
            self.files_translated += 1

        if not doc.translation_exists:
            self.files_missing_translation += 1

        if doc.translation_is_outdated:
            self.files_outdated += 1

    def first_outdated_files(self, lenght: int = 10) -> list[DocFile]:
        return list(filter(lambda d: d.translation_is_outdated, self.files))[
            :lenght
        ]

    def first_missing_translation_files(
        self, lenght: int = 10
    ) -> list[DocFile]:
        return list(filter(lambda d: not d.translation_exists, self.files))[
            :lenght
        ]



import datetime

from git import Repo


class FastAPIGitDocs:

    def __init__(self) -> None:
        self.repo = Repo(".")

    def get_commit_date_for(self, file: str) -> datetime.datetime | None:
        commits = list(self.repo.iter_commits(paths=file, max_count=1))
        if commits:
            return commits[0].committed_datetime

        return None



import datetime

from fastapi_translations.models import DocFile, Summary


def test_must_compute_summary_when_adding_docfile():
    doc1 = DocFile(
        translation_lang="es",
        original_file="/test/file1.md",
        original_commit=datetime.datetime(2024, 1, 1, 0, 0, 0),
        translation_file="/test/file1.md",
        translation_exists=True,
        translation_commit=datetime.datetime(2024, 1, 15, 0, 0, 0),
        translation_is_outdated=True
    )
    doc2 = DocFile(
        translation_lang="es",
        original_file="/test/file2.md",
        original_commit=datetime.datetime(2024, 1, 1, 0, 0, 0),
        translation_file="/test/file2.md",
        translation_exists=True,
        translation_commit=datetime.datetime(2024, 1, 1, 0, 0, 0),
        translation_is_outdated=False
    )
    doc3 = DocFile(
        translation_lang="es",
        original_file="/test/file3.md",
        original_commit=datetime.datetime(2024, 1, 1, 0, 0, 0),
        translation_file=None,
        translation_exists=False,
        translation_commit=None,
        translation_is_outdated=False
    )
    summary = Summary(lang="es")
    summary.append_file(doc1)
    summary.append_file(doc2)
    summary.append_file(doc3)

    assert summary.files_analyzed == 3
    assert summary.files_missing_translation == 1
    assert summary.files_outdated == 1
    assert summary.files_translated == 2
